@@ -41,6 +41,7 @@
 #endif
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <cairo-gobject.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkx.h>
 #include <gdk/gdkx.h>
@@ -95,6 +96,7 @@ typedef struct
     GtkTreeModel *model;
     GtkTreeIter *iter;
     GdkPixbuf *pix;
+    gint scale_factor;
 } PreviewData;
 
 typedef struct
@@ -155,6 +157,7 @@ typedef struct
 enum
 {
     COL_PIX = 0,
+    COL_SURFACE,
     COL_NAME,
     COL_FILENAME,
     COL_THUMBNAIL,
@@ -237,12 +240,12 @@ xfdesktop_settings_do_single_preview(PreviewData *pdata)
     if(thumbnail == NULL) {
         XF_DEBUG("generating thumbnail for filename %s", filename);
         pix = gdk_pixbuf_new_from_file_at_scale(filename,
-                                                PREVIEW_WIDTH, PREVIEW_HEIGHT,
+                                                PREVIEW_WIDTH * pdata->scale_factor, PREVIEW_HEIGHT * pdata->scale_factor,
                                                 TRUE, NULL);
     } else {
         XF_DEBUG("loading thumbnail %s", thumbnail);
         pix = gdk_pixbuf_new_from_file_at_scale(thumbnail,
-                                                PREVIEW_WIDTH, PREVIEW_HEIGHT,
+                                                PREVIEW_WIDTH * pdata->scale_factor, PREVIEW_HEIGHT * pdata->scale_factor,
                                                 TRUE, NULL);
         g_free(thumbnail);
     }
@@ -250,14 +253,20 @@ xfdesktop_settings_do_single_preview(PreviewData *pdata)
     g_free(filename);
 
     if(pix) {
+        cairo_surface_t *surface;
+
         pdata->pix = pix;
 
         /* set the image */
+        surface = gdk_cairo_surface_create_from_pixbuf(pix, pdata->scale_factor, NULL);
         gtk_list_store_set(GTK_LIST_STORE(pdata->model), pdata->iter,
-                       COL_PIX, pdata->pix,
-                       -1);
+                           COL_PIX, pdata->pix,
+                           COL_SURFACE, surface,
+                           -1);
+        cairo_surface_destroy(surface);
+
     }
-        xfdesktop_settings_free_pdata(pdata);
+    xfdesktop_settings_free_pdata(pdata);
 }
 
 static gboolean
@@ -336,6 +345,7 @@ cb_thumbnail_ready(XfdesktopThumbnailer *thumbnailer,
                 pdata->model = GTK_TREE_MODEL(g_object_ref(G_OBJECT(model)));
                 pdata->iter = gtk_tree_iter_copy(&iter);
                 pdata->pix = NULL;
+                pdata->scale_factor = gtk_widget_get_scale_factor(GTK_WIDGET(panel->image_iconview));
 
                 /* Create the preview image */
                 xfdesktop_settings_add_file_to_queue(panel, pdata);
@@ -365,6 +375,7 @@ xfdesktop_settings_queue_preview(GtkTreeModel *model,
         pdata = g_new0(PreviewData, 1);
         pdata->model = GTK_TREE_MODEL(g_object_ref(G_OBJECT(model)));
         pdata->iter = gtk_tree_iter_copy(iter);
+        pdata->scale_factor = gtk_widget_get_scale_factor(GTK_WIDGET(panel->image_iconview));
 
         XF_DEBUG("Thumbnailing failed, adding %s manually.", filename);
         xfdesktop_settings_add_file_to_queue(panel, pdata);
@@ -411,42 +422,35 @@ setup_special_icon_list(GtkBuilder *gxml,
     GtkTreeIter iter, parent_iter, child_iter;
     const struct {
         const gchar *name;
-        const gchar *icon;
-        const gchar *icon_fallback;
+        const gchar *icon_names[2];
         const gchar *xfconf_property;
         gboolean state;
     } icons[] = {
-        { N_("Home"), "user-home", "gnome-fs-desktop",
+        { N_("Home"), { "user-home", "gnome-fs-desktop" },
           DESKTOP_ICONS_SHOW_HOME, TRUE },
-        { N_("File System"), "drive-harddisk", "gnome-dev-harddisk",
+        { N_("File System"), { "drive-harddisk", "gnome-dev-harddisk" },
           DESKTOP_ICONS_SHOW_FILESYSTEM, TRUE },
-        { N_("Trash"), "user-trash", "gnome-fs-trash-empty",
+        { N_("Trash"), { "user-trash", "gnome-fs-trash-empty" },
           DESKTOP_ICONS_SHOW_TRASH, TRUE },
-        { N_("Removable Devices"), "drive-removable-media", "gnome-dev-removable",
+        { N_("Removable Devices"), { "drive-removable-media", "gnome-dev-removable" },
           DESKTOP_ICONS_SHOW_REMOVABLE, TRUE },
-        { N_("Network Shares"), "gtk-network", "gnome-dev-network",
+        { N_("Network Shares"), { "gtk-network", "gnome-dev-network" },
           DESKTOP_ICONS_SHOW_NETWORK_REMOVABLE, TRUE },
-        { N_("Disks and Drives"), "drive-harddisk-usb", "gnome-dev-removable-usb",
+        { N_("Disks and Drives"), { "drive-harddisk-usb", "gnome-dev-removable-usb" },
           DESKTOP_ICONS_SHOW_DEVICE_REMOVABLE, TRUE },
-        { N_("Other Devices"), "multimedia-player", "phone",
+        { N_("Other Devices"), { "multimedia-player", "phone" },
           DESKTOP_ICONS_SHOW_UNKNWON_REMOVABLE, TRUE },
-        { NULL, NULL, NULL, NULL, FALSE },
+        { NULL, { NULL, NULL }, NULL, FALSE },
     };
     const int REMOVABLE_DEVICES = 4;
     int i, w;
-    GtkIconTheme *itheme = gtk_icon_theme_get_default();
 
     gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, NULL);
 
-    ts = gtk_tree_store_new(N_ICON_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING,
+    ts = gtk_tree_store_new(N_ICON_COLS, G_TYPE_ICON, G_TYPE_STRING,
                             G_TYPE_BOOLEAN, G_TYPE_STRING);
     for(i = 0; icons[i].name; ++i) {
-        GdkPixbuf *pix = NULL;
-
-        if(gtk_icon_theme_has_icon(itheme, icons[i].icon))
-            pix = gtk_icon_theme_load_icon(itheme, icons[i].icon, w, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
-        else
-            pix = gtk_icon_theme_load_icon(itheme, icons[i].icon_fallback, w, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+        GIcon *icon = g_themed_icon_new_from_names((char **)icons[i].icon_names, G_N_ELEMENTS(icons[i].icon_names));
 
         if(i < REMOVABLE_DEVICES) {
             gtk_tree_store_append(ts, &parent_iter, NULL);
@@ -458,15 +462,15 @@ setup_special_icon_list(GtkBuilder *gxml,
 
         gtk_tree_store_set(ts, &iter,
                            COL_ICON_NAME, _(icons[i].name),
-                           COL_ICON_PIX, pix,
+                           COL_ICON_PIX, icon,
                            COL_ICON_PROPERTY, icons[i].xfconf_property,
                            COL_ICON_ENABLED,
                            xfconf_channel_get_bool(channel,
                                                    icons[i].xfconf_property,
                                                    icons[i].state),
                            -1);
-        if(pix)
-            g_object_unref(G_OBJECT(pix));
+        if (icon != NULL)
+            g_object_unref(icon);
     }
 
     treeview = GTK_WIDGET(gtk_builder_get_object(gxml, "treeview_default_icons"));
@@ -484,7 +488,7 @@ setup_special_icon_list(GtkBuilder *gxml,
 
     render = gtk_cell_renderer_pixbuf_new();
     gtk_tree_view_column_pack_start(col, render, FALSE);
-    gtk_tree_view_column_add_attribute(col, render, "pixbuf", COL_ICON_PIX);
+    gtk_tree_view_column_add_attribute(col, render, "gicon", COL_ICON_PIX);
 
     render = gtk_cell_renderer_text_new();
     gtk_tree_view_column_pack_start(col, render, TRUE);
@@ -646,13 +650,22 @@ xfdesktop_image_list_add_item(gpointer user_data)
     gtk_icon_view_set_model(GTK_ICON_VIEW(panel->image_iconview),
                             GTK_TREE_MODEL(dir_data->ls));
 
+    /* if we've actually switched to a new directory, select the first item
+     * in the model, else we won't end up saving the new directory, and it'll
+     * revert to whatever previous image/directory was set. */
+    if (dir_data->selected_iter == NULL) {
+        GtkTreeIter first;
+        gtk_tree_model_get_iter_first(GTK_TREE_MODEL(dir_data->ls), &first);
+        dir_data->selected_iter = gtk_tree_iter_copy(&first);
+    }
+
     /* last_image is in the directory added then it should be selected */
     if(dir_data->selected_iter) {
         GtkTreePath *path;
         path = gtk_tree_model_get_path(GTK_TREE_MODEL(dir_data->ls), dir_data->selected_iter);
+        gtk_tree_iter_free(dir_data->selected_iter);
         if(path) {
             gtk_icon_view_select_path(GTK_ICON_VIEW(panel->image_iconview), path);
-            gtk_tree_iter_free(dir_data->selected_iter);
             gtk_tree_path_free(path);
         }
      }
@@ -673,7 +686,7 @@ xfdesktop_image_list_add_dir(GObject *source_object,
 
     dir_data->panel = panel;
 
-    dir_data->ls = gtk_list_store_new(N_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING,
+    dir_data->ls = gtk_list_store_new(N_COLS, GDK_TYPE_PIXBUF, CAIRO_GOBJECT_TYPE_SURFACE, G_TYPE_STRING,
                                       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
     /* Get the last image/current image displayed so we can select it in the
@@ -899,8 +912,7 @@ cb_image_selection_changed(GtkIconView *icon_view,
         xfconf_channel_set_string(panel->channel, buf, filename);
     }
 
-    g_list_foreach (selected_items, (GFunc)gtk_tree_path_free, NULL);
-    g_list_free(selected_items);
+    g_list_free_full(selected_items, xfdesktop_tree_path_free);
     g_free(current_filename);
     if(buf)
         g_free(buf);
@@ -1062,10 +1074,8 @@ cb_xfdesktop_spin_icon_size_changed(GtkSpinButton *button,
     g_object_set_data(G_OBJECT(button), "xfconf-chanel", user_data);
 
     timer_id = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(button), "timer-id"));
-    if(timer_id != 0) {
+    if(timer_id != 0)
         g_source_remove(timer_id);
-        timer_id = 0;
-    }
 
     timer_id = g_timeout_add(500, xfdesktop_spin_icon_size_timer, button);
 
@@ -1118,42 +1128,40 @@ cb_folder_selection_changed(GtkWidget *button,
                             gpointer user_data)
 {
     AppearancePanel *panel = user_data;
-    gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(button));
-    gchar *previous_filename = NULL;
+    gchar *new_folder = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(button));
+    gchar *previous_folder = NULL;
 
     TRACE("entering");
 
     if(panel->selected_folder != NULL)
-        previous_filename = g_file_get_path(panel->selected_folder);
+        previous_folder = g_file_get_path(panel->selected_folder);
 
     /* Check to see if the folder actually did change */
-    if(g_strcmp0(filename, previous_filename) == 0) {
-        const gchar *current_folder = xfdesktop_settings_get_backdrop_image(panel);
-        gchar *dirname;
-
-        dirname = g_path_get_dirname(current_folder);
+    if(g_strcmp0(new_folder, previous_folder) == 0) {
+        const gchar *current_filename = xfdesktop_settings_get_backdrop_image(panel);
+        gchar *current_folder = g_path_get_dirname(current_filename);
 
         /* workaround another gtk bug - if the user sets the file chooser
          * button to something then it can't be changed with a set folder
          * call anymore. */
-        if(g_strcmp0(filename, dirname) == 0) {
+        if(g_strcmp0(new_folder, current_folder) == 0) {
             XF_DEBUG("folder didn't change");
-            g_free(dirname);
-            g_free(filename);
-            g_free(previous_filename);
+            g_free(current_folder);
+            g_free(new_folder);
+            g_free(previous_folder);
             return;
         } else {
-            g_free(filename);
-            filename = dirname;
+            g_free(new_folder);
+            new_folder = current_folder;
         }
     }
 
-    TRACE("folder changed to: %s", filename);
+    TRACE("folder changed to: %s", new_folder);
 
     if(panel->selected_folder != NULL)
         g_object_unref(panel->selected_folder);
 
-    panel->selected_folder = g_file_new_for_path(filename);
+    panel->selected_folder = g_file_new_for_path(new_folder);
 
     /* Stop any previous loading since something changed */
     xfdesktop_settings_stop_image_loading(panel);
@@ -1168,8 +1176,8 @@ cb_folder_selection_changed(GtkWidget *button,
                                     xfdesktop_image_list_add_dir,
                                     panel);
 
-    g_free(filename);
-    g_free(previous_filename);
+    g_free(new_folder);
+    g_free(previous_folder);
 }
 
 static void
@@ -1690,6 +1698,7 @@ static void
 xfdesktop_settings_setup_image_iconview(AppearancePanel *panel)
 {
     GtkIconView *iconview = GTK_ICON_VIEW(panel->image_iconview);
+    GList *cells;
 
     TRACE("entering");
 
@@ -1705,6 +1714,18 @@ xfdesktop_settings_setup_image_iconview(AppearancePanel *panel)
                 "margin", 2,
                 NULL);
 
+    cells = gtk_cell_layout_get_cells(GTK_CELL_LAYOUT(iconview));
+    for (GList *l = cells; l != NULL; l = l->next) {
+        if (GTK_IS_CELL_RENDERER_PIXBUF(l->data)) {
+            gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(iconview),
+                                           GTK_CELL_RENDERER(l->data),
+                                           "surface", COL_SURFACE,
+                                           NULL);
+            break;
+        }
+    }
+    g_list_free(cells);
+
     g_signal_connect(G_OBJECT(iconview), "selection-changed",
                      G_CALLBACK(cb_image_selection_changed), panel);
 
@@ -1802,7 +1823,7 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     /* tooltip options */
     chk_show_tooltips = GTK_WIDGET(gtk_builder_get_object(main_gxml, "chk_show_tooltips"));
     spin_tooltip_size = GTK_WIDGET(gtk_builder_get_object(main_gxml, "spin_tooltip_size"));
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_tooltip_size), 128);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_tooltip_size), 64);
 
     /* connect up the signals */
     g_signal_connect(G_OBJECT(chk_custom_font_size), "toggled",
@@ -1825,7 +1846,9 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
                                   TRUE);
 
     screen = gtk_widget_get_screen(appearance_container);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     wnck_screen = wnck_screen_get(panel->screen);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
     /* watch for workspace changes */
     g_signal_connect(wnck_screen, "workspace-created",
@@ -1975,6 +1998,10 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
 
 
     /* Menus Tab */
+    w = GTK_WIDGET(gtk_builder_get_object(main_gxml, "chk_show_delete_option"));
+    xfconf_g_property_bind(channel, DESKTOP_MENU_DELETE, G_TYPE_BOOLEAN,
+                           G_OBJECT(w), "active");
+
     w = GTK_WIDGET(gtk_builder_get_object(main_gxml, "chk_show_desktop_menu"));
     xfconf_g_property_bind(channel, SHOW_DESKTOP_MENU_PROP, G_TYPE_BOOLEAN,
                            G_OBJECT(w), "active");
@@ -2213,7 +2240,9 @@ main(int argc, char **argv)
         /* In a GtkPlug setting there isn't an easy way to find our window
          * in cb_window_opened so we'll just force wnck to init and get the
          * active window */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         wnck_screen = wnck_screen_get(screen);
+G_GNUC_END_IGNORE_DEPRECATIONS
         wnck_screen_force_update(wnck_screen);
         panel->wnck_window = wnck_screen_get_active_window(wnck_screen);
 
