@@ -57,15 +57,38 @@
 
 #include "xfdesktop-common.h"
 #include "xfdesktop-file-icon.h"
+#include "xfdesktop-file-manager-fdo-proxy.h"
 #include "xfdesktop-file-manager-proxy.h"
 #include "xfdesktop-file-utils.h"
 #include "xfdesktop-trash-proxy.h"
 #include "xfdesktop-thunar-proxy.h"
 
+typedef struct {
+    GtkWindow *parent;
+    GFile *file;
+} ExecuteData;
+
 static void xfdesktop_file_utils_add_emblems(GdkPixbuf *pix, GList *emblems);
 
 static XfdesktopTrash       *xfdesktop_file_utils_peek_trash_proxy(void);
 static XfdesktopFileManager *xfdesktop_file_utils_peek_filemanager_proxy(void);
+static XfdesktopFileManager1 *xfdesktop_file_utils_peek_filemanager_fdo_proxy(void);
+
+static void xfdesktop_file_utils_trash_proxy_new_cb (GObject *source_object,
+                                                     GAsyncResult *res,
+                                                     gpointer user_data);
+
+static void xfdesktop_file_utils_file_manager_proxy_new_cb (GObject *source_object,
+                                                            GAsyncResult *res,
+                                                            gpointer user_data);
+
+static void xfdesktop_file_utils_file_manager_fdo_proxy_new_cb(GObject *source_object,
+                                                               GAsyncResult *res,
+                                                               gpointer user_data);
+
+static void xfdesktop_file_utils_thunar_proxy_new_cb (GObject *source_object,
+                                                      GAsyncResult *res,
+                                                      gpointer user_data);
 
 #ifdef HAVE_THUNARX
 static XfdesktopThunar *xfdesktop_file_utils_peek_thunar_proxy(void);
@@ -244,6 +267,62 @@ xfdesktop_file_utils_get_display_name(GFile *file,
     return display_name;
 }
 
+/**
+ * xfdesktop_file_utils_next_new_file_name
+ * @filename : the filename which will be used as the basis/default
+ * @folder : the directory to search for a free filename
+ *
+ * Returns a filename that is like @filename with the possible addition of
+ * a number to differentiate it from other similarly named files. In other words
+ * it searches @folder for incrementally named files starting from @file_name
+ * and returns the first available increment.
+ *
+ * e.g. in a folder with the following files:
+ * - file
+ * - empty
+ * - file_copy
+ *
+ * Calling this functions with the above folder and @filename equal to 'file' the returned
+ * filename will be 'file (copy 1)'.
+ *
+ * The caller is responsible to free the returned string using g_free() when no longer needed.
+ *
+ * Code extracted and adapted from on thunar_util_next_new_file_name.
+ *
+ * Return value: pointer to the new filename.
+ **/
+gchar*
+xfdesktop_file_utils_next_new_file_name(const gchar *filename,
+                                        const gchar *folder)
+{
+  unsigned long   file_name_size  = strlen(filename);
+  unsigned        count           = 0;
+  gboolean        found_duplicate = FALSE;
+  gchar          *extension       = NULL;
+  gchar          *new_name        = g_strdup(filename);
+
+  extension = strrchr(filename, '.');
+  if (!extension || extension == filename)
+    extension = "";
+  else
+    file_name_size -= strlen(extension);
+
+  /* loop until new_name is unique */
+  while(TRUE)
+    {
+      GFile *file = g_file_new_build_filename(folder, new_name, NULL);
+      found_duplicate = g_file_query_exists(file, NULL);
+      g_object_unref(file);
+
+      if (!found_duplicate)
+        break;
+      g_free(new_name);
+      new_name = g_strdup_printf(_("%.*s (copy %u)%s"), (int) file_name_size, filename, ++count, extension ? extension : "");
+    }
+
+  return new_name;
+}
+
 GList *
 xfdesktop_file_utils_file_icon_list_to_file_list(GList *icon_list)
 {
@@ -319,8 +398,7 @@ xfdesktop_file_utils_file_list_to_uri_array(GList *file_list)
 void
 xfdesktop_file_utils_file_list_free(GList *file_list)
 {
-  g_list_foreach(file_list, (GFunc) g_object_unref, NULL);
-  g_list_free(file_list);
+    g_list_free_full(file_list, g_object_unref);
 }
 
 static GdkPixbuf *xfdesktop_fallback_icon = NULL;
@@ -686,12 +764,14 @@ xfdesktop_file_utils_async_handle_error(GError *error, gpointer userdata)
     GtkWindow *parent = GTK_WINDOW(userdata);
 
     if(error != NULL) {
-        xfce_message_dialog(parent,
-                            _("Error"), "dialog-error",
-                            _("The requested operation could not be completed"),
-                            error->message,
-                            XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
-                            NULL);
+        if(error->domain != G_IO_ERROR || error->code != G_IO_ERROR_TIMED_OUT) {
+            xfce_message_dialog(parent,
+                                _("Error"), "dialog-error",
+                                _("The requested operation could not be completed"),
+                                error->message,
+                                XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
+                                NULL);
+        }
 
         g_clear_error(&error);
     }
@@ -1096,6 +1176,16 @@ xfdesktop_file_utils_create_file_from_template(GFile *parent_folder,
     }
 }
 
+static void
+show_properties_fdo_cb(GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+    GError *error = NULL;
+    if (!xfdesktop_file_manager1_call_show_item_properties_finish(XFDESKTOP_FILE_MANAGER1(source_object), res, &error)) {
+        xfdesktop_file_utils_async_handle_error(error, user_data);
+    }
+}
+
+
 static void
 show_properties_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -1105,27 +1195,52 @@ show_properties_cb (GObject *source_object, GAsyncResult *res, gpointer user_dat
 }
 
 void
-xfdesktop_file_utils_show_properties_dialog(GFile *file,
+xfdesktop_file_utils_show_properties_dialog(GList *files,
                                             GdkScreen *screen,
                                             GtkWindow *parent)
 {
     XfdesktopFileManager *fileman_proxy;
+    XfdesktopFileManager1 *fileman_fdo_proxy;
 
-    g_return_if_fail(G_IS_FILE(file));
+    g_return_if_fail(files != NULL);
     g_return_if_fail(GDK_IS_SCREEN(screen) || GTK_IS_WINDOW(parent));
 
     if(!screen)
         screen = gtk_widget_get_screen(GTK_WIDGET(parent));
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
-    if(fileman_proxy) {
+    fileman_fdo_proxy = xfdesktop_file_utils_peek_filemanager_fdo_proxy();
+
+    if ((files->next != NULL || fileman_proxy == NULL) && fileman_fdo_proxy != NULL) {  // multiple files or no xfce fileman proxy
+        gchar **uris = g_new0(gchar *, g_list_length(files) + 1);
+        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
+        gint i = 0;
+
+        for (GList *l = files; l != NULL; l = l->next, ++i) {
+            uris[i] = g_file_get_uri(G_FILE(l->data));
+        }
+
+        xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
+
+        xfdesktop_file_manager1_call_show_item_properties(fileman_fdo_proxy,
+                                                          (const gchar *const *)uris,
+                                                          startup_id,
+                                                          NULL,
+                                                          show_properties_fdo_cb,
+                                                          parent);
+
+        xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
+
+        g_strfreev(uris);
+        g_free(startup_id);
+    } else if (files->next == NULL && fileman_proxy) {
+        GFile *file = files->data;
         gchar *uri = g_file_get_uri(file);
         gchar *display_name = g_strdup(gdk_display_get_name(gdk_screen_get_display(screen)));
         gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-
         xfdesktop_file_manager_call_display_file_properties(fileman_proxy,
                                                             uri, display_name, startup_id,
                                                             NULL,
@@ -1209,6 +1324,40 @@ xfdesktop_file_utils_launch(GFile *file,
     }
 }
 
+static void
+execute_finished_cb(GObject *source,
+                    GAsyncResult *res,
+                    gpointer user_data)
+{
+    ExecuteData *edata = (ExecuteData *)user_data;
+    gboolean ret;
+    GError *error = NULL;
+
+    ret = xfdesktop_file_manager_call_execute_finish(XFDESKTOP_FILE_MANAGER(source), res, &error);
+    if (!ret) {
+        gchar *filename = g_file_get_uri(edata->file);
+        gchar *name = g_filename_display_basename(filename);
+        gchar *primary = g_markup_printf_escaped(_("Failed to run \"%s\""), name);
+
+        xfce_message_dialog(edata->parent,
+                            _("Launch Error"), "dialog-error",
+                            primary, error->message,
+                            XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
+                            NULL);
+
+        g_free(primary);
+        g_free(name);
+        g_free(filename);
+        g_error_free(error);
+    }
+
+    if (edata->parent != NULL) {
+        g_object_unref(edata->parent);
+    }
+    g_object_unref(edata->file);
+    g_free(edata);
+}
+
 gboolean
 xfdesktop_file_utils_execute(GFile *working_directory,
                              GFile *file,
@@ -1229,8 +1378,9 @@ xfdesktop_file_utils_execute(GFile *working_directory,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
+        ExecuteData *edata;
         gchar *working_dir = working_directory != NULL ? g_file_get_uri(working_directory) : NULL;
+        const gchar *path_prop;
         gchar *uri = g_file_get_uri(file);
         gchar *display_name = g_strdup(gdk_display_get_name(gdk_screen_get_display(screen)));
         gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
@@ -1256,7 +1406,11 @@ xfdesktop_file_utils_execute(GFile *working_directory,
                 if(path != NULL) {
                     rc = xfce_rc_simple_open(path, TRUE);
                     if(rc != NULL) {
-                        working_dir = g_strdup(xfce_rc_read_entry(rc, "Path", NULL));
+                        path_prop = xfce_rc_read_entry(rc, "Path", NULL);
+                        if(xfce_str_is_empty(path_prop))
+                            working_dir = g_strdup(g_get_user_special_dir(G_USER_DIRECTORY_DESKTOP));
+                        else
+                            working_dir = g_strdup(path_prop);
                         xfce_rc_close(rc);
                     }
                     g_free(path);
@@ -1267,29 +1421,20 @@ xfdesktop_file_utils_execute(GFile *working_directory,
                 g_object_unref(info);
         }
 
-        if(!xfdesktop_file_manager_call_execute_sync(fileman_proxy,
-                                                     working_dir, uri,
-                                                     (const gchar **)uris,
-                                                     display_name, startup_id,
-                                                     NULL, &error))
-        {
-            gchar *filename = g_file_get_uri(file);
-            gchar *name = g_filename_display_basename(filename);
-            gchar *primary = g_markup_printf_escaped(_("Failed to run \"%s\""), name);
-
-            xfce_message_dialog(parent,
-                                _("Launch Error"), "dialog-error",
-                                primary, error->message,
-                                XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_free(primary);
-            g_free(name);
-            g_free(filename);
-            g_clear_error(&error);
-
-            success = FALSE;
+        edata = g_new0(ExecuteData, 1);
+        if (parent != NULL) {
+            edata->parent = g_object_ref(parent);
         }
+        edata->file = g_object_ref(file);
+        xfdesktop_file_manager_call_execute(fileman_proxy,
+                                            working_dir,
+                                            uri,
+                                            (const gchar **)uris,
+                                            display_name,
+                                            startup_id,
+                                            NULL,
+                                            execute_finished_cb,
+                                            edata);
 
         g_free(startup_id);
         g_strfreev(uris);
@@ -1323,15 +1468,16 @@ static void
 display_chooser_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
     GError *error = NULL;
-    if (!xfdesktop_file_manager_call_display_chooser_dialog_finish(XFDESKTOP_FILE_MANAGER(source_object), res, &error))
+    if (!xfdesktop_file_manager_call_display_application_chooser_dialog_finish(XFDESKTOP_FILE_MANAGER(source_object), res, &error))
         xfdesktop_file_utils_async_handle_error(error, user_data);
 }
 
 void
-xfdesktop_file_utils_display_chooser_dialog(GFile *file,
-                                            gboolean open,
-                                            GdkScreen *screen,
-                                            GtkWindow *parent)
+xfdesktop_file_utils_display_app_chooser_dialog(GFile *file,
+                                                gboolean open,
+                                                gboolean preselect_default_checkbox,
+                                                GdkScreen *screen,
+                                                GtkWindow *parent)
 {
     XfdesktopFileManager *fileman_proxy;
 
@@ -1349,13 +1495,14 @@ xfdesktop_file_utils_display_chooser_dialog(GFile *file,
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        xfdesktop_file_manager_call_display_chooser_dialog(fileman_proxy,
-                                                           uri, open,
-                                                           display_name,
-                                                           startup_id,
-                                                           NULL,
-                                                           display_chooser_cb,
-                                                           parent);
+        xfdesktop_file_manager_call_display_application_chooser_dialog(fileman_proxy,
+                                                                       uri, open,
+                                                                       preselect_default_checkbox,
+                                                                       display_name,
+                                                                       startup_id,
+                                                                       NULL,
+                                                                       display_chooser_cb,
+                                                                       parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -1373,78 +1520,53 @@ xfdesktop_file_utils_display_chooser_dialog(GFile *file,
     }
 }
 
+/**
+ * 'out_source_files' will hold a list owned by caller with contents owned by callee
+ * 'dest_source_files' will hold a list and contents owned by caller
+ */
 void
-xfdesktop_file_utils_transfer_file(GdkDragAction action,
-                                   GFile *source_file,
-                                   GFile *target_file,
-                                   GdkScreen *screen)
+xfdesktop_file_utils_build_transfer_file_lists(GdkDragAction action,
+                                               GList *source_icons,
+                                               XfdesktopFileIcon *dest_icon,
+                                               GList **out_source_files,
+                                               GList **out_dest_files)
 {
-    XfdesktopFileManager *fileman_proxy;
+    g_return_if_fail(source_icons != NULL);
+    g_return_if_fail(XFDESKTOP_IS_FILE_ICON(dest_icon));
+    g_return_if_fail(out_source_files != NULL && out_dest_files != NULL);
 
-    g_return_if_fail(G_IS_FILE(source_file));
-    g_return_if_fail(G_IS_FILE(target_file));
-    g_return_if_fail(screen == NULL || GDK_IS_SCREEN(screen));
+    switch (action) {
+        case GDK_ACTION_MOVE:
+        case GDK_ACTION_LINK:
+            *out_dest_files = g_list_append(NULL, g_object_ref(xfdesktop_file_icon_peek_file(dest_icon)));
+            for (GList *l = source_icons; l != NULL; l = l->next) {
+                GFile *source_file = xfdesktop_file_icon_peek_file(XFDESKTOP_FILE_ICON(l->data));
+                if (source_file != NULL) {
+                    *out_source_files = g_list_prepend(*out_source_files, source_file);
+                }
+            }
+            *out_source_files = g_list_reverse(*out_source_files);
+            break;
 
-    if(!screen)
-        screen = gdk_display_get_default_screen(gdk_display_get_default());
+        case GDK_ACTION_COPY:
+            for (GList *l = source_icons; l != NULL; l = l->next) {
+                GFile *source_file = xfdesktop_file_icon_peek_file(XFDESKTOP_FILE_ICON(l->data));
+                if (source_file != NULL) {
+                    gchar *name = g_file_get_basename(source_file);
+                    if (name != NULL) {
+                        GFile *dest_file = g_file_get_child(xfdesktop_file_icon_peek_file(dest_icon), name);
+                        *out_dest_files = g_list_prepend(*out_dest_files, dest_file);
+                        *out_source_files = g_list_prepend(*out_source_files, source_file);
+                        g_free(name);
+                    }
+                }
+            }
+            *out_source_files = g_list_reverse(*out_source_files);
+            *out_dest_files = g_list_reverse(*out_dest_files);
+            break;
 
-    fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
-    if(fileman_proxy) {
-        GError *error = NULL;
-        gchar *source_uris[2] = { g_file_get_uri(source_file), NULL };
-        gchar *target_uris[2] = { g_file_get_uri(target_file), NULL };
-        gchar *display_name = g_strdup(gdk_display_get_name(gdk_screen_get_display(screen)));
-        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
-
-        switch(action) {
-            case GDK_ACTION_MOVE:
-                xfdesktop_file_manager_call_move_into_sync(fileman_proxy, "",
-                                                           (const gchar **)source_uris,
-                                                           (const gchar *)target_uris[0],
-                                                           display_name, startup_id,
-                                                           NULL, &error);
-                break;
-            case GDK_ACTION_COPY:
-                xfdesktop_file_manager_call_copy_to_sync(fileman_proxy, "",
-                                                         (const gchar **)source_uris,
-                                                         (const gchar **)target_uris,
-                                                         display_name, startup_id,
-                                                         NULL, &error);
-                break;
-            case GDK_ACTION_LINK:
-                xfdesktop_file_manager_call_link_into_sync(fileman_proxy, "",
-                                                           (const gchar **)source_uris,
-                                                           (const gchar *)target_uris[0],
-                                                           display_name, startup_id,
-                                                           NULL, &error);
-                break;
-            default:
-                g_warning("Unsupported transfer action");
-        }
-
-        if(error) {
-            xfce_message_dialog(NULL,
-                                _("Transfer Error"), "dialog-error",
-                                _("The file transfer could not be performed"),
-                                error->message,
-                                XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_clear_error(&error);
-        }
-
-        g_free(startup_id);
-        g_free(display_name);
-        g_free(target_uris[0]);
-        g_free(source_uris[0]);
-    } else {
-        xfce_message_dialog(NULL,
-                            _("Transfer Error"), "dialog-error",
-                            _("The file transfer could not be performed"),
-                            _("This feature requires a file manager service to "
-                              "be present (such as the one supplied by Thunar)."),
-                            XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
-                            NULL);
+        default:
+            g_warning("Unsupported drag action: %d", action);
     }
 }
 
@@ -1515,8 +1637,8 @@ xfdesktop_file_utils_transfer_files(GdkDragAction action,
 
         g_free(startup_id);
         g_free(display_name);
-        g_free(target_uris[0]);
-        g_free(source_uris[0]);
+        g_strfreev(target_uris);
+        g_strfreev(source_uris);
     } else {
         xfce_message_dialog(NULL,
                             _("Transfer Error"), "dialog-error",
@@ -1536,6 +1658,7 @@ static gint dbus_ref_cnt = 0;
 static GDBusConnection *dbus_gconn = NULL;
 static XfdesktopTrash *dbus_trash_proxy = NULL;
 static XfdesktopFileManager *dbus_filemanager_proxy = NULL;
+static XfdesktopFileManager1 *dbus_filemanager_fdo_proxy = NULL;
 #ifdef HAVE_THUNARX
 static XfdesktopThunar *dbus_thunar_proxy = NULL;
 #else
@@ -1554,27 +1677,38 @@ xfdesktop_file_utils_dbus_init(void)
     }
 
     if(dbus_gconn) {
-        dbus_trash_proxy = xfdesktop_trash_proxy_new_sync(dbus_gconn,
-                                                          G_DBUS_PROXY_FLAGS_NONE,
-                                                          "org.xfce.FileManager",
-                                                          "/org/xfce/FileManager",
-                                                          NULL,
-                                                          NULL);
+        xfdesktop_trash_proxy_new(dbus_gconn,
+                                  G_DBUS_PROXY_FLAGS_NONE,
+                                  "org.xfce.FileManager",
+                                  "/org/xfce/FileManager",
+                                  NULL,
+                                  xfdesktop_file_utils_trash_proxy_new_cb,
+                                  NULL);
 
-        dbus_filemanager_proxy = xfdesktop_file_manager_proxy_new_sync(dbus_gconn,
-                                                                       G_DBUS_PROXY_FLAGS_NONE,
-                                                                       "org.xfce.FileManager",
-                                                                       "/org/xfce/FileManager",
-                                                                       NULL,
-                                                                       NULL);
+        xfdesktop_file_manager_proxy_new(dbus_gconn,
+                                         G_DBUS_PROXY_FLAGS_NONE,
+                                         "org.xfce.FileManager",
+                                         "/org/xfce/FileManager",
+                                         NULL,
+                                         xfdesktop_file_utils_file_manager_proxy_new_cb,
+                                         NULL);
+
+        xfdesktop_file_manager1_proxy_new(dbus_gconn,
+                                         G_DBUS_PROXY_FLAGS_NONE,
+                                         "org.freedesktop.FileManager1",
+                                         "/org/freedesktop/FileManager1",
+                                         NULL,
+                                         xfdesktop_file_utils_file_manager_fdo_proxy_new_cb,
+                                         NULL);
 
 #ifdef HAVE_THUNARX
-        dbus_thunar_proxy = xfdesktop_thunar_proxy_new_sync(dbus_gconn,
-                                                            G_DBUS_PROXY_FLAGS_NONE,
-                                                            "org.xfce.FileManager",
-                                                            "/org/xfce/FileManager",
-                                                            NULL,
-                                                            NULL);
+        xfdesktop_thunar_proxy_new(dbus_gconn,
+                                   G_DBUS_PROXY_FLAGS_NONE,
+                                   "org.xfce.FileManager",
+                                   "/org/xfce/FileManager",
+                                   NULL,
+                                   xfdesktop_file_utils_thunar_proxy_new_cb,
+                                   NULL);
 #else
         dbus_thunar_proxy = NULL;
 #endif
@@ -1599,6 +1733,12 @@ xfdesktop_file_utils_peek_filemanager_proxy(void)
     return dbus_filemanager_proxy;
 }
 
+static XfdesktopFileManager1 *
+xfdesktop_file_utils_peek_filemanager_fdo_proxy(void)
+{
+    return dbus_filemanager_fdo_proxy;
+}
+
 #ifdef HAVE_THUNARX
 static XfdesktopThunar *
 xfdesktop_file_utils_peek_thunar_proxy(void)
@@ -1613,6 +1753,37 @@ xfdesktop_file_utils_peek_thunar_proxy(void)
 }
 #endif
 
+static void
+xfdesktop_file_utils_trash_proxy_new_cb (GObject *source_object,
+                                         GAsyncResult *res,
+                                         gpointer user_data) {
+    dbus_trash_proxy = xfdesktop_trash_proxy_new_finish (res, NULL);
+}
+
+static void
+xfdesktop_file_utils_file_manager_proxy_new_cb (GObject *source_object,
+                                                GAsyncResult *res,
+                                                gpointer user_data) {
+    dbus_filemanager_proxy = xfdesktop_file_manager_proxy_new_finish (res, NULL);
+}
+
+static void
+xfdesktop_file_utils_file_manager_fdo_proxy_new_cb(GObject *source_object,
+                                                   GAsyncResult *res,
+                                                   gpointer user_data)
+{
+    dbus_filemanager_fdo_proxy = xfdesktop_file_manager1_proxy_new_finish(res, NULL);
+}
+
+static void
+xfdesktop_file_utils_thunar_proxy_new_cb (GObject *source_object,
+                                          GAsyncResult *res,
+                                          gpointer user_data) {
+#ifdef HAVE_THUNARX
+    dbus_thunar_proxy = xfdesktop_thunar_proxy_new_finish (res, NULL);
+#endif
+}
+
 void
 xfdesktop_file_utils_dbus_cleanup(void)
 {
@@ -1623,6 +1794,9 @@ xfdesktop_file_utils_dbus_cleanup(void)
         g_object_unref(G_OBJECT(dbus_trash_proxy));
     if(dbus_filemanager_proxy)
         g_object_unref(G_OBJECT(dbus_filemanager_proxy));
+    if (dbus_filemanager_fdo_proxy) {
+        g_object_unref(G_OBJECT(dbus_filemanager_fdo_proxy));
+    }
     if(dbus_thunar_proxy)
         g_object_unref(G_OBJECT(dbus_thunar_proxy));
     if(dbus_gconn)
